package com.winhc.task.run;

import com.mongodb.client.FindIterable;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.model.Filters;
import com.winhc.task.util.BaseUtils;
import com.winhc.task.util.ESUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.bson.Document;
import org.bson.conversions.Bson;
import org.frameworkset.elasticsearch.boot.BBossESStarter;
import org.frameworkset.elasticsearch.client.ClientInterface;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.List;

import static com.mongodb.client.model.Filters.in;

/**
 * @author π
 * @Description:定时删除任务
 * @date 2021/6/22 17:07
 */

@Component
@Slf4j
@EnableScheduling
@AllArgsConstructor
@ConditionalOnProperty(prefix = "scheduling", name = "enabled", havingValue = "true")
public class CleanDataTask {
    @Autowired
    @Qualifier("bbossESStarterEs5")
    private BBossESStarter bbossESStarterEs5;
    @Autowired
    @Qualifier("bbossESStarterEs6")
    private BBossESStarter bbossESStarterEs6;

    private ClientInterface restClient5;
    private ClientInterface restClient6;

    private final MongoTemplate mongoTemplate;


    @PostConstruct
    public void init() {
        restClient5 = bbossESStarterEs5.getRestClient("es5");
        restClient6 = bbossESStarterEs5.getRestClient("es6");
    }

    //@Scheduled(cron = "*/30 * * * * ?")
    //@Scheduled(cron = "00 00 06 * * ?")
    @Scheduled(cron = "0 0/10 * * * ?")
    public void start() throws InterruptedException {
        log.info("startJob CleanDataTask !!! ");

        String wenshu_mongo = "wenshu_remove_user";
        String case_mongo = "sifa_remove_user";
        int batchSize = 200;

        String wenshu_index = "wenshu_detail2";
        String judicial_case_index = "winhc_judicial_case_v2";
        String judicial_case_detail_index = "winhc_judicial_case_detail_v2";

//        String wenshu_index = "test_wenshu";
//        String judicial_case_index = "test_case1";
//        String judicial_case_detail_index = "test_case2";

        //先查询mongo再删除es文书
        step1(wenshu_mongo, wenshu_index, batchSize);
        //先查询mongo再删除es司法案件
        step2(case_mongo, judicial_case_index, judicial_case_detail_index, batchSize);

        log.info("stopJob CleanDataTask !!! ");

    }

    private void step2(String case_mongo, String judicial_case_index, String judicial_case_detail_index, int batchSize) throws InterruptedException {
        ArrayList<String> caseList = new ArrayList<>();
        for (Document document : mongoTemplate.getCollection(case_mongo).find().noCursorTimeout(true).batchSize(batchSize)) {
            String case_no = BaseUtils.case_no_trim(document.getString("_id"));
            if (StringUtils.isNotBlank(case_no)) {
                caseList.add(case_no);
            }
            if (caseList.size() >= batchSize) {
                caseDeleted(judicial_case_index, judicial_case_detail_index, caseList);
            }
        }
        if (!caseList.isEmpty()) {
            caseDeleted(judicial_case_index, judicial_case_detail_index, caseList);
        }
    }

    private void step1(String wenshu_mongo, String wenshu_index, int batchSize) throws InterruptedException {
        ArrayList<String> idList = new ArrayList<>();
        for (Document document : mongoTemplate.getCollection(wenshu_mongo).find().noCursorTimeout(true).batchSize(batchSize)) {
            String id = document.getString("_id");
            if (StringUtils.isNotBlank(id)) {
                idList.add(id);
            }
            if (idList.size() >= batchSize) {
                wenshuUpdate(wenshu_index, idList);
            }
        }
        if (!idList.isEmpty()) {
            wenshuUpdate(wenshu_index, idList);
        }
    }

    /**
     * 物理删除司法案件
     *
     * @param judicial_case_index
     * @param judicial_case_detail_index
     * @param caseList
     * @throws InterruptedException
     */
    private void caseDeleted(String judicial_case_index, String judicial_case_detail_index, ArrayList<String> caseList) throws InterruptedException {
        String res2 = restClient6.deleteByQuery(judicial_case_index + "/_delete_by_query?conflicts=proceed&wait_for_completion=false&refresh=true", ESUtils.deletedByQuery(caseList));
        String res3 = restClient6.deleteByQuery(judicial_case_detail_index + "/_delete_by_query?conflicts=proceed&wait_for_completion=false&refresh=true", ESUtils.deletedByQuery(caseList));
        log.info("caseDeleted1 res : {}\n", res2);
        log.info("caseDeleted2 res : {}\n", res3);
        caseList.clear();
        Thread.sleep(1000 * 2);
    }

    /**
     * 逻辑删除文书
     *
     * @param wenshu_index
     * @param idList
     * @throws InterruptedException
     */
    private void wenshuUpdate(String wenshu_index, ArrayList<String> idList) throws InterruptedException {
        String res1 = restClient5.updateByQuery(wenshu_index + "/_update_by_query?conflicts=proceed&wait_for_completion=false&refresh=true", ESUtils.updateWenshuById(idList));
        log.info("wenshuUpdate res : {}\n", res1);
        idList.clear();
        Thread.sleep(1000 * 2);
    }
}
